// Copyright (c) Microsoft Corporation.  All rights reserved.

using System;
using System.Windows;
using System.Windows.Controls;
using Microsoft.WindowsAPICodePack.Shell;
using Microsoft.WindowsAPICodePack.Dialogs;
using Microsoft.WindowsAPICodePack.Shell.PropertySystem;
using System.Collections.Generic;
using System.Windows.Threading;

namespace Microsoft.WindowsAPICodePack.Samples.SearchApp
{
    /// <summary>
    /// Interaction logic for AdvancedSearch.xaml
    /// </summary>
    public partial class AdvancedSearch : Window
    {
        private StockIcons stockIcons;
        private StockIcon documentsStockIcon;
        private StockIcon picturesStockIcon;
        private StockIcon musicStockIcon;
        private StockIcon videosStockIcon;

        internal Window1 MainWindow;

        public AdvancedSearch()
        {
            stockIcons = new StockIcons();

            documentsStockIcon = stockIcons.DocumentAssociated;
            videosStockIcon = stockIcons.VideoFiles;
            musicStockIcon = stockIcons.AudioFiles;
            picturesStockIcon = stockIcons.ImageFiles;

            InitializeComponent();

            // Set our default
            DocumentsRadioButton.IsChecked = true;

            // 
            prop1prop2OperationComboBox.SelectedIndex = 0;
        }

        private void DocumentsRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            UpdateDocumentsSearchSettings();

            documentsStockIcon.Selected = true;
            DocumentsRadioButton.Content = new Image { Source = documentsStockIcon.BitmapSource };

            picturesStockIcon.Selected = false;
            PicturesRadioButton.Content = new Image { Source = picturesStockIcon.BitmapSource };

            musicStockIcon.Selected = false;
            MusicRadioButton.Content = new Image { Source = musicStockIcon.BitmapSource };

            videosStockIcon.Selected = false;
            VideosRadioButton.Content = new Image { Source = videosStockIcon.BitmapSource };
        }

        private void PicturesRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            UpdatePicturesSearchSettings();

            documentsStockIcon.Selected = false;
            DocumentsRadioButton.Content = new Image { Source = documentsStockIcon.BitmapSource };

            picturesStockIcon.Selected = true;
            PicturesRadioButton.Content = new Image { Source = picturesStockIcon.BitmapSource };

            musicStockIcon.Selected = false;
            MusicRadioButton.Content = new Image { Source = musicStockIcon.BitmapSource };

            videosStockIcon.Selected = false;
            VideosRadioButton.Content = new Image { Source = videosStockIcon.BitmapSource };
        }

        private void MusicRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            UpdateMusicSearchSettings();

            documentsStockIcon.Selected = false;
            DocumentsRadioButton.Content = new Image { Source = documentsStockIcon.BitmapSource };

            picturesStockIcon.Selected = false;
            PicturesRadioButton.Content = new Image { Source = picturesStockIcon.BitmapSource };

            musicStockIcon.Selected = true;
            MusicRadioButton.Content = new Image { Source = musicStockIcon.BitmapSource };

            videosStockIcon.Selected = false;
            VideosRadioButton.Content = new Image { Source = videosStockIcon.BitmapSource };
        }

        private void VideosRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            UpdateVideosSearchSettings();

            documentsStockIcon.Selected = false;
            DocumentsRadioButton.Content = new Image { Source = documentsStockIcon.BitmapSource };

            picturesStockIcon.Selected = false;
            PicturesRadioButton.Content = new Image { Source = picturesStockIcon.BitmapSource };

            musicStockIcon.Selected = false;
            MusicRadioButton.Content = new Image { Source = musicStockIcon.BitmapSource };

            videosStockIcon.Selected = true;
            VideosRadioButton.Content = new Image { Source = videosStockIcon.BitmapSource };
        }

        private void UpdateDocumentsSearchSettings()
        {
            // We are in "documents" mode
            prop1ComboBox.Items.Clear();
            prop1ComboBox.Items.Add("Author");
            prop1ComboBox.Items.Add("Title");
            prop1ComboBox.Items.Add("Comment");
            prop1ComboBox.Items.Add("Copyright");
            prop1ComboBox.Items.Add("Pages");
            prop1ComboBox.Items.Add("Tags/Keywords");
            prop1ComboBox.SelectedIndex = 0;

            prop1OperationComboBox.SelectedIndex = 9;

            prop1TextBox.Text = "";

            prop2ComboBox.Items.Clear();
            prop2ComboBox.Items.Add("Author");
            prop2ComboBox.Items.Add("Title");
            prop2ComboBox.Items.Add("Comment");
            prop2ComboBox.Items.Add("Copyright");
            prop2ComboBox.Items.Add("Pages");
            prop2ComboBox.Items.Add("Tags/Keywords");
            prop2ComboBox.SelectedIndex = 5;

            prop2OperationComboBox.SelectedIndex = 0;

            prop2TextBox.Text = "";

            prop1prop2OperationComboBox.SelectedIndex = 0;

            comboBoxDateCreated.SelectedIndex = 0;

            // locations
            locationsListBox.Items.Clear();

            if(ShellLibrary.IsPlatformSupported)
                AddLocation((ShellContainer)KnownFolders.DocumentsLibrary);
            else
                AddLocation((ShellContainer)KnownFolders.Documents);
        }

        private void UpdatePicturesSearchSettings()
        {
            // We are in "Pictures" mode
            prop1ComboBox.Items.Clear();
            prop1ComboBox.Items.Add("Author");
            prop1ComboBox.Items.Add("Subject");
            prop1ComboBox.Items.Add("Camera maker");
            prop1ComboBox.Items.Add("Copyright");
            prop1ComboBox.Items.Add("Rating");
            prop1ComboBox.Items.Add("Tags/Keywords");
            prop1ComboBox.SelectedIndex = 0;

            prop1OperationComboBox.SelectedIndex = 9;

            prop1TextBox.Text = "";

            prop2ComboBox.Items.Clear();
            prop2ComboBox.Items.Add("Author");
            prop2ComboBox.Items.Add("Subject");
            prop2ComboBox.Items.Add("Camera maker");
            prop2ComboBox.Items.Add("Copyright");
            prop2ComboBox.Items.Add("Rating");
            prop2ComboBox.Items.Add("Tags/Keywords");
            prop2ComboBox.SelectedIndex = 0;

            prop2OperationComboBox.SelectedIndex = 0;

            prop2TextBox.Text = "";

            prop1prop2OperationComboBox.SelectedIndex = 0;

            comboBoxDateCreated.SelectedIndex = 0;

            // locations
            locationsListBox.Items.Clear();

            if (ShellLibrary.IsPlatformSupported)
                AddLocation((ShellContainer)KnownFolders.PicturesLibrary);
            else
                AddLocation((ShellContainer)KnownFolders.Pictures);

        }

        private void UpdateMusicSearchSettings()
        {
            // We are in "Music" mode
            prop1ComboBox.Items.Clear();
            prop1ComboBox.Items.Add("Album artist");
            prop1ComboBox.Items.Add("Album title");
            prop1ComboBox.Items.Add("Composer");
            prop1ComboBox.Items.Add("Rating");
            prop1ComboBox.Items.Add("Genre");
            prop1ComboBox.Items.Add("Year");
            prop1ComboBox.SelectedIndex = 1;

            prop1OperationComboBox.SelectedIndex = 9;

            prop1TextBox.Text = "";

            prop2ComboBox.Items.Clear();
            prop2ComboBox.Items.Add("Album artist");
            prop2ComboBox.Items.Add("Album title");
            prop2ComboBox.Items.Add("Composer");
            prop2ComboBox.Items.Add("Rating");
            prop2ComboBox.Items.Add("Genre");
            prop2ComboBox.Items.Add("Year");
            prop2ComboBox.SelectedIndex = 0;

            prop2OperationComboBox.SelectedIndex = 0;

            prop2TextBox.Text = "";

            prop1prop2OperationComboBox.SelectedIndex = 0;

            comboBoxDateCreated.SelectedIndex = 0;

            // locations
            locationsListBox.Items.Clear();

            if (ShellLibrary.IsPlatformSupported)
                AddLocation((ShellContainer)KnownFolders.MusicLibrary);
            else
                AddLocation((ShellContainer)KnownFolders.Music);
        }

        private void UpdateVideosSearchSettings()
        {
            // We are in "Videos" mode
            prop1ComboBox.Items.Clear();
            prop1ComboBox.Items.Add("Title");
            prop1ComboBox.Items.Add("Video length");
            prop1ComboBox.Items.Add("Comment");
            prop1ComboBox.SelectedIndex = 0;

            prop1OperationComboBox.SelectedIndex = 9;

            prop1TextBox.Text = "";

            prop2ComboBox.Items.Clear();
            prop1ComboBox.Items.Add("Title");
            prop1ComboBox.Items.Add("Video length");
            prop1ComboBox.Items.Add("Comment");
            prop2ComboBox.SelectedIndex = 0;

            prop2OperationComboBox.SelectedIndex = 0;

            prop2TextBox.Text = "";

            prop1prop2OperationComboBox.SelectedIndex = 0;

            comboBoxDateCreated.SelectedIndex = 0;

            // locations
            locationsListBox.Items.Clear();

            if (ShellLibrary.IsPlatformSupported)
                AddLocation((ShellContainer)KnownFolders.VideosLibrary);
            else
                AddLocation((ShellContainer)KnownFolders.Videos);
        }

        private void addLocationButton_Click(object sender, RoutedEventArgs e)
        {
            // Show CFD and let users pick a folder
            CommonOpenFileDialog cfd = new CommonOpenFileDialog();
            cfd.AllowNonFileSystemItems = true;
            cfd.IsFolderPicker = true;
            cfd.Multiselect = true;

            if (cfd.ShowDialog() == CommonFileDialogResult.OK)
            {
                // Loop through each "folder" and add it our list
                foreach (ShellContainer so in cfd.FilesAsShellObject)
                {
                    AddLocation(so);
                }
            }
        }

        private void AddLocation(ShellContainer so)
        {
            StackPanel sp = new StackPanel();
            sp.Orientation = Orientation.Horizontal;

            // Add the thumbnail/icon
            Image img = new Image();
            img.Source = so.Thumbnail.SmallBitmapSource;
            img.Margin = new Thickness(5);
            sp.Children.Add(img);

            // Add the name/title
            TextBlock tb = new TextBlock();
            tb.Text = so.Name;
            tb.Margin = new Thickness(5);
            sp.Children.Add(tb);

            // Set our tag as the shell container user picked...
            sp.Tag = so;

            //
            locationsListBox.Items.Add(sp);
        }

        private void removeLocationButton_Click(object sender, RoutedEventArgs e)
        {
            if (locationsListBox.SelectedItem != null)
                locationsListBox.Items.Remove(locationsListBox.SelectedItem);
        }

        private void buttonClear_Click(object sender, RoutedEventArgs e)
        {
            // Reset all the settings on the dialog for the selected search type
            if (DocumentsRadioButton.IsChecked.Value)
                UpdateDocumentsSearchSettings();
            else if (PicturesRadioButton.IsChecked.Value)
                UpdatePicturesSearchSettings();
            else if (VideosRadioButton.IsChecked.Value)
                UpdateVideosSearchSettings();
            else if (MusicRadioButton.IsChecked.Value)
                UpdateMusicSearchSettings();
        }

        private DateTime ParseDate(string toParse, DateTime relativeDate)
        {
            if (string.IsNullOrEmpty(toParse) || !toParse.StartsWith("date:"))
                throw new ArgumentException();

            string tmpToParse = toParse.ToLower().Replace("date:", "");

            switch (tmpToParse)
            {
                case "a long time ago":
                    DateTime longTimeAgo = new DateTime(relativeDate.Year - 2, 1, 1, 0, 0, 0);
                    return longTimeAgo;
                case "earlier this year":
                    DateTime thisYear = new DateTime(relativeDate.Year, 1, 1, 0, 0, 0);
                    return thisYear;
                case "earlier this month":
                    DateTime thisMonth = new DateTime(relativeDate.Year, relativeDate.Month, 1, 0, 0, 0);
                    return thisMonth;
                case "last week":
                    DayOfWeek dayOfWeek = relativeDate.DayOfWeek;
                    DateTime lastweekSunday = relativeDate.AddDays(-1 * (int)dayOfWeek);
                    return lastweekSunday;
                case "yesterday":
                    DateTime yesterday = relativeDate.AddDays(-1);
                    return yesterday;
                case "earlier this week":
                    DateTime lastWeek = relativeDate.AddDays(-7);
                    return lastWeek;
                default:
                    throw new ArgumentException();
            }
        }

        private void buttonSearch_Click(object sender, RoutedEventArgs e)
        {
            if (MainWindow == null)
                return;

            // This is our final searchcondition that we'll create the search folder from
            SearchCondition finalSearchCondition = null;

            // This is Prop1 + prop2 search condition... if the user didn't specify one of the properties,
            // we can just use the one property/value they specify...if they do, then we can do the and/or operation
            SearchCondition combinedPropSearchCondition = null;

            // If we have a valid first property/value, then create a search condition
            if (!string.IsNullOrEmpty(prop1TextBox.Text))
            {
                SearchCondition prop1Condition = SearchConditionFactory.CreateLeafCondition(
                    GetSearchProperty(prop1ComboBox.SelectedValue.ToString()),
                    prop1TextBox.Text,
                    GetConditionOperation(prop1OperationComboBox));

                // After creating the first condition, see if we need to create a second leaf condition
                if (prop1prop2OperationComboBox.SelectedIndex != 0 &&
                    !(string.IsNullOrEmpty(prop2TextBox.Text)))
                {
                    SearchCondition prop2Condition = SearchConditionFactory.CreateLeafCondition(
                        GetSearchProperty(prop2ComboBox.SelectedValue.ToString()),
                        prop2TextBox.Text,
                        GetConditionOperation(prop2OperationComboBox));

                    // Create our combined search condition AND or OR
                    if (prop1prop2OperationComboBox.SelectedIndex == 1)
                        combinedPropSearchCondition = SearchConditionFactory.CreateAndOrCondition(
                            SearchConditionType.And,
                            false, prop1Condition, prop2Condition);
                    else
                        combinedPropSearchCondition = SearchConditionFactory.CreateAndOrCondition(
                            SearchConditionType.Or,
                            false, prop1Condition, prop2Condition);
                }
                else
                    combinedPropSearchCondition = prop1Condition;
            }
            else
                return; // no search text entered

            // Get the date condition
            if (comboBoxDateCreated.SelectedItem != dateCreatedNone)
            {
                SearchCondition dateCreatedCondition = SearchConditionFactory.CreateLeafCondition(
                    SystemProperties.System.DateCreated,
                    ParseDate(((ComboBoxItem)comboBoxDateCreated.SelectedItem).Tag.ToString(), DateTime.Now),
                    SearchConditionOperation.GreaterThan);

                // If we have a property based search condition, create an "AND" search condition from these 2
                if (combinedPropSearchCondition != null)
                    finalSearchCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.And,
                        false, combinedPropSearchCondition, dateCreatedCondition);
                else
                    finalSearchCondition = dateCreatedCondition;
            }
            else
                finalSearchCondition = combinedPropSearchCondition;

            // Get our search scopes
            List<ShellContainer> scopes = new List<ShellContainer>();

            foreach (StackPanel sp in locationsListBox.Items)
            {
                if (sp.Tag is ShellContainer)
                    scopes.Add((ShellContainer)sp.Tag);
            }

            ShellSearchFolder searchFolder = new ShellSearchFolder(finalSearchCondition, scopes.ToArray());

            // Set our stacks to show
            searchFolder.SetStacks(GetStacks());

            // Search via the main window's UI
            MainWindow.searchFolder = searchFolder;

            // Perform the search on a background thread so we don't block the main UI
            MainWindow.Search();
        }

        private PropertyKey GetSearchProperty(string prop)
        {
            switch (prop)
            {
                case "Author":
                    return SystemProperties.System.Author;
                case "Title":
                    return SystemProperties.System.Title;
                case "Comment":
                    return SystemProperties.System.Comment;
                case "Copyright":
                    return SystemProperties.System.Copyright;
                case "Pages":
                    return SystemProperties.System.Document.PageCount;
                case "Tags/Keywords":
                    return SystemProperties.System.Keywords;
                case "Subject":
                    return SystemProperties.System.Subject;
                case "Camera maker":
                    return SystemProperties.System.Photo.CameraManufacturer;
                case "Rating":
                    return SystemProperties.System.Rating;
                case "Album artist":
                    return SystemProperties.System.Music.AlbumArtist;
                case "Album title":
                    return SystemProperties.System.Music.AlbumTitle;
                case "Composer":
                    return SystemProperties.System.Music.Composer;
                case "Genre":
                    return SystemProperties.System.Music.Genre;
                case "Video length":
                    return SystemProperties.System.Media.Duration;
                case "Year":
                    return SystemProperties.System.Media.Year;
            }

            return SystemProperties.System.Null;
        }

        private string[] GetStacks()
        {
            List<string> stacks = new List<string>();

            foreach (ListBoxItem lbi in stackToShowListBox.SelectedItems)
                stacks.Add(lbi.Tag.ToString());

            return stacks.ToArray();
        }

        private SearchConditionOperation GetConditionOperation(ComboBox comboBox)
        {
            SearchConditionOperation operation = (SearchConditionOperation)Enum.Parse(
                typeof(SearchConditionOperation),
                ((ComboBoxItem)comboBox.Items[comboBox.SelectedIndex]).Tag.ToString(),
                true);

            return operation;
        }

        private void prop1prop2OperationComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Based on the And/OR operation between the two properties, enable/disable
            // the second propertie's UI
            if (prop1prop2OperationComboBox.SelectedIndex == 0) // (None)
            {
                prop2ComboBox.IsEnabled = false;
                prop2OperationComboBox.IsEnabled = false;
                prop2TextBox.IsEnabled = false;
            }
            else
            {
                prop2ComboBox.IsEnabled = true;
                prop2OperationComboBox.IsEnabled = true;
                prop2TextBox.IsEnabled = true;
            }
        }
    }
}
