﻿'Copyright (c) Microsoft Corporation.  All rights reserved.


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Controls.Primitives
Imports System.Windows.Threading
Imports Microsoft.WindowsAPICodePack.Shell
Imports System.Windows.Input
Imports Microsoft.WindowsAPICodePack.Shell.PropertySystem
Imports System.Windows.Media
Imports System.Windows.Interop

Namespace Microsoft.WindowsAPICodePack.Samples.SearchApp
	''' <summary>
	''' Interaction logic for Window1.xaml
	''' </summary>
	Partial Public Class Window1
		Inherits Window
        Private top_Renamed As Integer = 10, left_Renamed As Integer = 10 ' for glass
		Private neverRendered As Boolean = True ' for glass
		Private Const WM_DWMCOMPOSITIONCHANGED As Integer = &H031E ' for glass (when DWM / glass setting is changed)
		Friend searchFolder As ShellSearchFolder = Nothing
		Private advWindow As AdvancedSearch ' keep only one instance of the advanced window (unless user closes it)

		Private currentIndex As Integer = -1 ' for our navigation history
		Private navigationHistory As New List(Of ShellContainer)()

		Public Sub New()
			InitializeComponent()
			AddHandler DragThumb.DragDelta, AddressOf OnMove

			AddHandler SourceInitialized, AddressOf Window1_SourceInitialized
			AddHandler Loaded, AddressOf Window1_Loaded

			' Handle double clicks on the ListView items - 
			' for our search app, this is particularly useful if we have stacks showing up in the result
			' and the user wants to drill down.
			' Or if the search results contains folders...
			AddHandler listView1.MouseDoubleClick, AddressOf listView1_MouseDoubleClick
		End Sub

		Private Sub Window1_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Dim source As HwndSource = HwndSource.FromHwnd(New WindowInteropHelper(Me).Handle)
			source.AddHook(New HwndSourceHook(AddressOf WndProc))
		End Sub

		Private Sub listView1_MouseDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs)
			' When the user tries to navigate a ShellContainer,
			' change our ListView's ItemsSource to the new ShellContainer object.
			If listView1.SelectedItem IsNot Nothing AndAlso TypeOf listView1.SelectedItem Is ShellContainer Then
				' Add to our navigation history
				navigationHistory.Add(CType(listView1.SelectedItem, ShellContainer))
				currentIndex += 1

				' Update the listview's itemsource
				listView1.ItemsSource = CType(listView1.SelectedItem, ShellContainer)

				If listView1.Items.Count > 0 Then
					listView1.SelectedIndex = 0
				End If

				UpdateNavButtons()
			End If
		End Sub

		Friend Sub Search()
			Dim items As New List(Of ShellObject)()
			For Each so As ShellObject In searchFolder
				items.Add(so)
			Next so

			' Update the listview's itemsource
			listView1.ItemsSource = items

			If listView1.Items.Count > 0 Then
				listView1.SelectedIndex = 0
			End If

			' Add to our navigation history
			navigationHistory.Add(searchFolder)
			currentIndex += 1

			UpdateNavButtons()
		End Sub

		#Region "For the Aero glass effect"

		Private Function WndProc(ByVal hwnd As IntPtr, ByVal msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr, ByRef handled As Boolean) As IntPtr
			' handle the message for DWM when the aero glass is turned on or off
			If msg = WM_DWMCOMPOSITIONCHANGED Then
				If GlassHelper.IsGlassEnabled Then
					' Extend glass
                    Dim bounds As Rect = VisualTreeHelper.GetDescendantBounds(listView1)
					GlassHelper.ExtendGlassFrame(Me, New Thickness(bounds.Left, bounds.Top, bounds.Right, bounds.Bottom))
				Else
					' turn off glass...
					GlassHelper.DisableGlassFrame(Me)
				End If

				handled = True
			End If

			Return IntPtr.Zero
		End Function

		Private Sub OnMove(ByVal s As Object, ByVal e As DragDeltaEventArgs)
			left_Renamed += CInt(Fix(e.HorizontalChange))
			top_Renamed += CInt(Fix(e.VerticalChange))
			Me.Left = left_Renamed
			Me.Top = top_Renamed
		End Sub

		Private Sub Window1_SourceInitialized(ByVal sender As Object, ByVal e As EventArgs)
            Dim bounds As Rect = VisualTreeHelper.GetDescendantBounds(listView1)
            GlassHelper.ExtendGlassFrame(Me, New Thickness(bounds.Left, bounds.Top, bounds.Right, bounds.Bottom))
        End Sub

		Protected Overrides Sub OnContentRendered(ByVal e As EventArgs)
			If Me.neverRendered Then
				' The window takes the size of its content because SizeToContent
				' is set to WidthAndHeight in the markup. We then allow
				' it to be set by the user, and have the content take the size
				' of the window.
				Me.SizeToContent = SizeToContent.Manual

				Dim root As FrameworkElement = TryCast(Me.Content, FrameworkElement)
				If root IsNot Nothing Then
					root.Width = Double.NaN
					root.Height = Double.NaN
				End If

				Me.neverRendered = False
			End If

			MyBase.OnContentRendered(e)
		End Sub


		#End Region

		Private Sub ButtonBack_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If currentIndex > 0 AndAlso ((currentIndex - 1) > -1) Then
				currentIndex -= 1

				' Update the listview's itemsource
				listView1.ItemsSource = navigationHistory(currentIndex)

				If listView1.Items.Count > 0 Then
					listView1.SelectedIndex = 0
				End If
			End If

			UpdateNavButtons()
		End Sub

		Private Sub ButtonForward_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If currentIndex >= 0 AndAlso ((currentIndex + 1) < navigationHistory.Count) Then
				currentIndex += 1

				' Update the listview's itemsource
				listView1.ItemsSource = navigationHistory(currentIndex)

				If listView1.Items.Count > 0 Then
					listView1.SelectedIndex = 0
				End If
			End If

			UpdateNavButtons()
		End Sub

		Private Sub UpdateNavButtons()
			If currentIndex <= 0 Then
				ButtonBack.IsEnabled = False
			ElseIf navigationHistory.Count > 1 AndAlso (currentIndex > 0) Then
				ButtonBack.IsEnabled = True
			End If

			If navigationHistory.Count > 1 AndAlso (currentIndex < navigationHistory.Count - 1) Then
				ButtonForward.IsEnabled = True
			Else
				ButtonForward.IsEnabled = False
			End If
		End Sub

		Private Sub SearchTextBox_Search(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Search... on any letters typed
			If Not String.IsNullOrEmpty(SearchBox.Text) Then
				Dim combinedPropertyCondition As SearchCondition = GetPropertyCondition(SearchBox.Text)

				Dim dateCondition As SearchCondition = SearchConditionFactory.CreateLeafCondition(SystemProperties.System.Title, SearchBox.Text, SearchConditionOperation.ValueContains)

				Dim finalSearchCondition As SearchCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.Or, False, combinedPropertyCondition, dateCondition)

				' Create a new search folder by setting our search condition and couple of search scopes

				If ShellLibrary.IsPlatformSupported Then
					searchFolder = New ShellSearchFolder(finalSearchCondition, CType(KnownFolders.DocumentsLibrary, ShellContainer), CType(KnownFolders.PicturesLibrary, ShellContainer), CType(KnownFolders.VideosLibrary, ShellContainer))
				Else
					searchFolder = New ShellSearchFolder(finalSearchCondition, CType(KnownFolders.Documents, ShellContainer), CType(KnownFolders.Pictures, ShellContainer), CType(KnownFolders.Videos, ShellContainer))
				End If

                Search()
			Else
				listView1.ItemsSource = Nothing ' nothing was typed, or user deleted the search query (clear the list).
				currentIndex += 1
			End If
		End Sub

		Private Function GetPropertyCondition(ByVal searchText As String) As SearchCondition
			' Loop through each word user types and create a search condition
			Dim words() As String = searchText.Split(" "c)

			' This is what we'll return at the end
			Dim combinedPropertyCondition As SearchCondition = Nothing

			For Each word As String In words
				' Our first sample condition - compare the search query with the System.Filename
				Dim propertyCondition1 As SearchCondition = SearchConditionFactory.CreateLeafCondition(SystemProperties.System.FileName, word, SearchConditionOperation.ValueContains)

				' Second sample condition - compare the search query with the System.Keywords (tags)
				Dim propertyCondition2 As SearchCondition = SearchConditionFactory.CreateLeafCondition(SystemProperties.System.Keywords, word, SearchConditionOperation.ValueContains)

				Dim tmpCombinedCondition As SearchCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.Or, False, propertyCondition1, propertyCondition2)

				' If we already have an existing condition, do "AND" operation between these two
				If combinedPropertyCondition IsNot Nothing Then
					combinedPropertyCondition = SearchConditionFactory.CreateAndOrCondition(SearchConditionType.And, False, combinedPropertyCondition, tmpCombinedCondition)
				Else
					combinedPropertyCondition = tmpCombinedCondition
				End If
			Next word

			Return combinedPropertyCondition
		End Function

		Private Sub CommandBinding_Executed(ByVal sender As Object, ByVal e As ExecutedRoutedEventArgs)
			SearchBox.Focus()
		End Sub

		Private Sub buttonSearchAdv_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
            If advWindow Is Nothing Then
                advWindow = New AdvancedSearch()
                advWindow.MainWindow = Me
                AddHandler advWindow.Closed, AddressOf advWindow_Closed
            End If

			If Not advWindow.IsVisible Then
				advWindow.Show()
			Else
				advWindow.Visibility = Visibility.Visible
				advWindow.Focus()
			End If
		End Sub

		Private Sub advWindow_Closed(ByVal sender As Object, ByVal e As EventArgs)
			advWindow = Nothing
		End Sub

		Private Sub Window_Closed(ByVal sender As Object, ByVal e As EventArgs)
			If advWindow IsNot Nothing Then
				advWindow.Close()
			End If
		End Sub
	End Class

	''' <summary>
	''' ImageView displays image files using themselves as their icons.
	''' In order to write our own visual tree of a view, we should override its
	''' DefaultStyleKey and ItemContainerDefaultKey. DefaultStyleKey specifies
	''' the style key of ListView; ItemContainerDefaultKey specifies the style
	''' key of ListViewItem.
	''' </summary>
	Public Class ImageView
		Inherits ViewBase
		#Region "DefaultStyleKey"

		Protected Overrides ReadOnly Property DefaultStyleKey() As Object
			Get
				Return New ComponentResourceKey(Me.GetType(), "ImageView")
			End Get
		End Property

		#End Region

		#Region "ItemContainerDefaultStyleKey"

		Protected Overrides ReadOnly Property ItemContainerDefaultStyleKey() As Object
			Get
				Return New ComponentResourceKey(Me.GetType(), "ImageViewItem")
			End Get
		End Property

		#End Region
	End Class

	Public NotInheritable Class CustomCommands
		Public Shared SearchCommand As New RoutedCommand("SearchCommand", GetType(CustomCommands))
	End Class
End Namespace
