﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CommandUndo
{
    public partial class Form1 : Form
    {
        private CommandHistory _history = new CommandHistory();
        private List<IWidget> _widgets = new List<IWidget>();
        private bool _isDragging = false;
        private IWidget _dragWidget = null;
        private Point _prevMousePt;
        
        public Form1()
        {
            InitializeComponent();

            panelSurface.MouseDoubleClick += new MouseEventHandler(panelSurface_MouseDoubleClick);
            panelSurface.Paint += new PaintEventHandler(panelSurface_Paint);
            panelSurface.MouseMove += new MouseEventHandler(panelSurface_MouseMove);
            panelSurface.MouseDown += new MouseEventHandler(panelSurface_MouseDown);
            panelSurface.MouseUp += new MouseEventHandler(panelSurface_MouseUp);

            editToolStripMenuItem.DropDownOpening += new EventHandler(editToolStripMenuItem_DropDownOpening);
            undoToolStripMenuItem.Click += new EventHandler(undoToolStripMenuItem_Click);
        }

        void panelSurface_MouseUp(object sender, MouseEventArgs e)
        {
            _isDragging = false;
            _dragWidget = null;
        }

        void panelSurface_MouseDown(object sender, MouseEventArgs e)
        {
            IWidget widget = GetWidgetUnderPoint(e.Location);
            if (widget != null)
            {
                _dragWidget = widget;
                _isDragging = true;
                _prevMousePt = e.Location;
            }
        }

        void panelSurface_MouseMove(object sender, MouseEventArgs e)
        {
            if (!_isDragging)
            {
                IWidget widget = GetWidgetUnderPoint(e.Location);
                if (widget != null)
                {
                    panelSurface.Cursor = Cursors.SizeAll;
                }
                else
                {
                    panelSurface.Cursor = Cursors.Default;
                }
            }
            else if (_dragWidget != null)
            {
                
                Size offset = new Size(e.Location.X - _prevMousePt.X,
                    e.Location.Y - _prevMousePt.Y);
                
                _prevMousePt = e.Location;
                
                RunCommand(new MoveCommand(_dragWidget, offset));
                
                Refresh();
            }
        }

        private void RunCommand(ICommand command)
        {
            _history.PushCommand(command);
            command.Execute(); 
        }

        private IWidget GetWidgetUnderPoint(Point point)
        {
            foreach (IWidget widget in _widgets)
            {
                if (widget.BoundingBox.Contains(point))
                {
                    return widget;
                }
            }
            return null;
        }

        void panelSurface_Paint(object sender, PaintEventArgs e)
        {
            foreach (IWidget widget in _widgets)
            {
                widget.Draw(e.Graphics);
            }
        }

        void panelSurface_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            _widgets.Add(new Widget(e.Location));
            Refresh();
        }


        void editToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            undoToolStripMenuItem.Enabled = _history.CanUndo;
            if (_history.CanUndo)
            {
                undoToolStripMenuItem.Text = "Undo " + _history.MostRecentCommandName;
            }
        }

        void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ICommand command = _history.PopCommand();
            command.Undo();
            Refresh();
        }

        
    }
}
